@@ -31,11 +31,11 @@
 #
 
 """Apply JSON-Patches according to
-http://tools.ietf.org/html/draft-pbryan-json-patch-04"""
+http://tools.ietf.org/html/draft-ietf-appsawg-json-patch-04"""
 
 # Will be parsed by setup.py to determine package metadata
 __author__ = 'Stefan Kögl <stefan@skoegl.net>'
-__version__ = '0.4'
+__version__ = '0.5'
 __website__ = 'https://github.com/stefankoegl/python-json-patch'
 __license__ = 'Modified BSD License'
 
@@ -63,6 +63,9 @@ class JsonPatchConflict(JsonPatchException):
     - etc.
     """
 
+class JsonPatchTestFailed(JsonPatchException, AssertionError):
+    """ A Test operation failed """
+
 
 def apply_patch(doc, patch, in_place=False):
     """Apply list of patches to specified json document.
@@ -169,7 +172,8 @@ class JsonPatch(object):
             'add': AddOperation,
             'replace': ReplaceOperation,
             'move': MoveOperation,
-            'test': TestOperation
+            'test': TestOperation,
+            'copy': CopyOperation,
         }
 
     def __str__(self):
@@ -411,6 +415,23 @@ class TestOperation(PatchOperation):
     """Test value by specified location."""
 
     def apply(self, obj):
-        value = self.operation['value']
+        try:
+            subobj, part = self.locate(obj, self.location)
+        except JsonPatchConflict, c:
+            raise JsonPatchTestFailed(str(c))
+
+        val = subobj[part]
+
+        if 'value' in self.operation:
+            value = self.operation['value']
+            if val != value:
+                raise JsonPatchTestFailed('%s is not equal to tested value %s' % (val, value))
+
+
+class CopyOperation(PatchOperation):
+    """ Copies an object property or an array element to a new location """
+
+    def apply(self, obj):
         subobj, part = self.locate(obj, self.location)
-        assert subobj[part] == value
+        value = copy.deepcopy(subobj[part])
+        AddOperation(self.operation['to'], {'value': value}).apply(obj)
